require 'uu/os/form/input'

module UU
  module OS
    module Form

      # Component for choosing color.
      class ColorPicker < UU::OS::Form::Input

        # Constant denoting palette containing standard uuOS colors.
        UU_OS_PALETTE = 'UU_OS_PALETTE'

        # Data containing palette of colors. If no data is supplied,
        # default palette is used containing colors from {Palette}.
        #
        # @example
        #   # default palette
        #   colorpicker.data = {:palette => UU::OS::Form::ColorPicker::UU_OS_PALETTE}
        #
        #   # custom palette
        #   colorpicker.data = {:palette => ["#ff0000", "#00ff00", "#0000ff"]}
        #
        #   # custom palette
        #   palette = UU::OS::Form::ColorPicker::Palette
        #   colorpicker.data = {:palette => [palette::YELLOW, palette::ORANGE]}
        attr_accessor :data

        # URI of a uuCommand to use for loading data. The data returned from
        # the uuCommand must be in a format supported by {#data data} attribute.
        attr_accessor :data_uri

        # Selected color in hexadecimal format, such as "#ff0000". It is possible
        # to set a pre-defined color from the {UU::OS::Form::ColorPicker::Palette}
        # or a custom color. Note that if a value that is not present in the used
        # palette is set, the palette offered to the user won't contain that color,
        # so if the user chooses yet another color, (s)he won't be able to get back
        # to the original one.
        #
        # @example
        #   # set custom color
        #   colorpicker.value = "#c0a5ca"
        #
        #   # set color from standard palette
        #   colorpicker.value = UU::OS::Form::ColorPicker::Palette::ORANGE
        #
        #   # get value and use it for coloring a button component (note that
        #   # class Palette is able to return color names only for colors from
        #   # default palette)
        #   color_hex = colorpicker.value
        #   button.style_class = UU::OS::Form::ColorPicker::Palette.to_hash().key(color_hex)
        attr_accessor :value

        def data
          attributes[:data]
        end

        def data=(value)
          attributes[:data] = value
        end

        def data_uri
          attributes[:dataUri]
        end

        def data_uri=(value)
          attributes[:dataUri] = value
        end

        def value
          attributes[:value]
        end

        def value=(value)
          attributes[:value] = value
        end

        # Color palette containing standard uuOS colors.
        class Palette
          GRAY = '#e5e5e5'
          STEEL = '#8e99a1'
          GOLD = '#bead73'
          DARK_VIOLET = '#7c6ec6'
          VIOLET = '#ac8cd0'
          PINK = '#e294ba'
          RED = '#de6060'
          ORANGE = '#eb9144'
          YELLOW = '#ebb128'
          LIGHT_GREEN = '#9cbd48'
          GREEN = '#69bc72'
          TURQUOISE = '#44c0a4'
          AZURE = '#61bddd'
          BLUE = '#58a2e6'
          DARK_BLUE = '#5275c1'

          # Returns hash containing camelCased color names as keys and color hex
          # codes as values. Color names can be used in other components,
          # e.g. Button, in their style_class attribute.
          #
          # @return [Hash] Hash containing camelCased color names as keys and
          #         color hex codes as values.
          # @example
          #   # button component colored according to the chosen color from
          #   # colorpicker using default (this) palette
          #   color_hex = colorpicker.value
          #   button.style_class = UU::OS::Form::ColorPicker::Palette.to_hash().key(color_hex)
          def self.to_hash
            h = {}
            Palette.constants.each do |c|
              key_camel_case = c.to_s.downcase.gsub(/_./) { |match| match.slice(1,1).upcase}
              h[key_camel_case] = Palette.const_get(c)
            end
            h
          end
        end

      end
    end
  end
end
